import { Autocomplete, Box, TextField } from "@mui/material";
import { Create, useAutocomplete } from "@refinedev/mui";
import { useForm } from "@refinedev/react-hook-form";
import { Controller } from "react-hook-form";
import { useEffect, useState } from "react";
import { EVERY_LANGUAGE, Languages, languageStore } from "@stores";
import { LanguageSwitch } from "@/components/LanguageSwitch";
import { axiosInstanceForGet } from "@/providers";
import { useNavigate } from "react-router";
import { useNotification } from "@refinedev/core";

export const CityCreate = () => {
  const { language, setLanguageAction } = languageStore;
  const navigate = useNavigate();
  const notification = useNotification();

  // State to manage city name translations across all supported languages.
  // Initializes with empty strings for each language.
  const [allLanguageNames, setAllLanguageNames] = useState<
    Record<Languages, string>
  >(EVERY_LANGUAGE(""));

  const {
    saveButtonProps,
    refineCore: { formLoading },
    register,
    control,
    setValue,
    watch,
    handleSubmit,
    formState: { errors },
  } = useForm<{
    name: string;
    country_code: string;
    arms: string;
  }>({});

  // Keeps the 'name' input field synchronized with the currently active language's translation.
  // Updates whenever the active language or the `allLanguageNames` state changes.
  useEffect(() => {
    setValue("name", allLanguageNames[language]);
  }, [language, allLanguageNames, setValue]);

  // Captures the current value of the 'name' TextField and updates the `allLanguageNames` state.
  // This is vital for preserving user input when switching languages or before form submission.
  const updateCurrentLanguageName = () => {
    const currentNameValue = watch("name");
    setAllLanguageNames((prev) => ({
      ...prev,
      [language]: currentNameValue || "",
    }));
  };

  // Handles language changes. It first saves the current input, then updates the active language.
  const handleLanguageChange = (lang: Languages) => {
    updateCurrentLanguageName();
    setLanguageAction(lang);
  };

  // Autocomplete hooks for selecting a country and city arms (media).
  const { autocompleteProps: countryAutocompleteProps } = useAutocomplete({
    resource: "country",
  });

  const { autocompleteProps: mediaAutocompleteProps } = useAutocomplete({
    resource: "media",
    onSearch: (value) => [
      {
        field: "media_name",
        operator: "contains",
        value,
      },
    ],
  });

  // --- Form Submission Logic ---

  // Handles the form submission. It saves the current language's input,
  // validates the Russian name, and then sends requests to create/update city data
  // across different languages.
  const onFinish = async (data: {
    name: string;
    country_code: string;
    arms: string;
  }) => {
    updateCurrentLanguageName();

    const finalNames = {
      ...allLanguageNames,
      [language]: data.name,
    };

    try {
      if (!finalNames.ru) {
        console.error("Russian name is required for initial city creation.");
        if (notification && typeof notification.open === "function") {
          notification.open({
            message: "Ошибка",
            description: "Русское название города обязательно для создания.",
            type: "error",
          });
        }
        return;
      }

      console.log("Submitting with names:", finalNames);

      // Create the city with the Russian name first.
      const ruResponse = await axiosInstanceForGet("ru").post("/city", {
        name: finalNames.ru,
        country_code: data.country_code,
        arms: data.arms,
      });

      const id = ruResponse.data.id;

      // Update the city with English and Chinese names if available.
      if (finalNames.en) {
        await axiosInstanceForGet("en").patch(`/city/${id}`, {
          name: finalNames.en,
          country_code: data.country_code,
          arms: data.arms,
        });
      }

      if (finalNames.zh) {
        await axiosInstanceForGet("zh").patch(`/city/${id}`, {
          name: finalNames.zh,
          country_code: data.country_code,
          arms: data.arms,
        });
      }

      console.log("City created/updated successfully!");
      if (notification && typeof notification.open === "function") {
        notification.open({
          message: "Город успешно создан",
          type: "success",
        });
      }

      navigate("/city", { replace: true });
    } catch (error) {
      console.error("Error creating/updating city:", error);
    }
  };

  return (
    <Create
      isLoading={formLoading}
      saveButtonProps={{
        ...saveButtonProps,
        disabled: saveButtonProps.disabled,
        onClick: handleSubmit(onFinish as any),
      }}
    >
      <Box
        component="form"
        sx={{ display: "flex", flexDirection: "column" }}
        autoComplete="off"
      >
        <LanguageSwitch action={handleLanguageChange} />

        <Controller
          control={control}
          name="country_code"
          rules={{ required: "Это поле является обязательным" }}
          defaultValue={null}
          render={({ field }) => (
            <Autocomplete
              {...countryAutocompleteProps}
              value={
                countryAutocompleteProps.options.find(
                  (option: { code: string; name: string; id: string }) =>
                    option.code === field.value
                ) || null
              }
              onChange={(
                _,
                value: { code: string; name: string; id: string } | null
              ) => {
                field.onChange(value?.code || "");
              }}
              getOptionLabel={(item: {
                code: string;
                name: string;
                id: string;
              }) => {
                return item ? item.name : "";
              }}
              isOptionEqualToValue={(
                option: { code: string; name: string; id: string },
                value: { code: string; name: string; id: string }
              ) => {
                return option.id === value?.id;
              }}
              renderInput={(params) => (
                <TextField
                  {...params}
                  label="Выберите страну"
                  margin="normal"
                  variant="outlined"
                  error={!!errors.country_code}
                  required
                />
              )}
            />
          )}
        />

        <TextField
          {...register("name", {
            required: "Это поле является обязательным",
            onBlur: updateCurrentLanguageName,
          })}
          error={!!errors.name}
          margin="normal"
          fullWidth
          InputLabelProps={{ shrink: true }}
          type="text"
          label={"Название *"}
          name="name"
        />

        <Controller
          control={control}
          name="arms"
          defaultValue={null}
          render={({ field }) => (
            <Autocomplete
              {...mediaAutocompleteProps}
              value={
                mediaAutocompleteProps.options.find(
                  (option: { id: string; media_name: string }) =>
                    option.id === field.value
                ) || null
              }
              onChange={(
                _,
                value: { id: string; media_name: string } | null
              ) => {
                field.onChange(value?.id || "");
              }}
              getOptionLabel={(item: { id: string; media_name: string }) => {
                return item ? item.media_name : "";
              }}
              isOptionEqualToValue={(
                option: { id: string; media_name: string },
                value: { id: string; media_name: string }
              ) => {
                return option.id === value?.id;
              }}
              filterOptions={(
                options: { id: string; media_name: string }[],
                { inputValue }
              ) => {
                return options.filter((option) =>
                  option.media_name
                    .toLowerCase()
                    .includes(inputValue.toLowerCase())
                );
              }}
              renderInput={(params) => (
                <TextField
                  {...params}
                  label="Выберите герб"
                  margin="normal"
                  variant="outlined"
                  error={!!errors.arms}
                />
              )}
            />
          )}
        />
      </Box>
    </Create>
  );
};
