import {
  Autocomplete,
  Box,
  TextField,
  Typography,
  FormControlLabel,
  Paper,
  Grid,
  Checkbox,
} from "@mui/material";
import { Edit, useAutocomplete } from "@refinedev/mui";
import { useForm } from "@refinedev/react-hook-form";
import { Controller } from "react-hook-form";

import { useParams } from "react-router";
import { LinkedItems } from "../../components/LinkedItems";
import { type SightItem, sightFields } from "./types";
import { observer } from "mobx-react-lite";
import { languageStore } from "../../store/LanguageStore";
import { useEffect, useState } from "react";
import { LanguageSwitch } from "../../components/LanguageSwitch/index";

const TRANSFER_FIELDS = [
  { name: "bus", label: "Автобус" },
  { name: "metro_blue", label: "Метро (синяя)" },
  { name: "metro_green", label: "Метро (зеленая)" },
  { name: "metro_orange", label: "Метро (оранжевая)" },
  { name: "metro_purple", label: "Метро (фиолетовая)" },
  { name: "metro_red", label: "Метро (красная)" },
  { name: "train", label: "Электричка" },
  { name: "tram", label: "Трамвай" },
  { name: "trolleybus", label: "Троллейбус" },
];

export const StationEdit = observer(() => {
  const { language, setLanguageAction } = languageStore;
  const [stationData, setStationData] = useState({
    ru: {
      name: "",
      system_name: "",
      description: "",
      address: "",
      latitude: 0,
      longitude: 0,
    },
    en: {
      name: "",
      system_name: "",
      description: "",
      address: "",
      latitude: 0,
      longitude: 0,
    },
    zh: {
      name: "",
      system_name: "",
      description: "",
      address: "",
      latitude: 0,
      longitude: 0,
    },
  });

  const handleLanguageChange = () => {
    setStationData((prevData) => ({
      ...prevData,
      [language]: {
        name: watch("name") ?? "",
        system_name: watch("system_name") ?? "",
        description: watch("description") ?? "",
        address: watch("address") ?? "",
        latitude: Number(watch("latitude")) || 0,
        longitude: Number(watch("longitude")) || 0,
      },
    }));
  };

  const [coordinatesPreview, setCoordinatesPreview] = useState({
    latitude: "",
    longitude: "",
  });

  const {
    saveButtonProps,
    register,
    control,
    getValues,
    setValue,
    watch,
    formState: { errors },
  } = useForm({
    refineCoreProps: {
      meta: {
        headers: { "Accept-Language": language },
      },
    },
  });

  const directions = [
    {
      label: "Прямой",
      value: true,
    },
    {
      label: "Обратный",
      value: false,
    },
  ];

  const directionContent = watch("direction");
  const [routeDirection, setRouteDirection] = useState(false);

  useEffect(() => {
    if (directionContent) {
      setRouteDirection(directionContent);
    }
  }, [directionContent]);

  useEffect(() => {
    if (stationData[language as keyof typeof stationData]?.name) {
      setValue("name", stationData[language as keyof typeof stationData]?.name);
    }
    if (stationData[language as keyof typeof stationData]?.system_name) {
      setValue(
        "system_name",
        stationData[language as keyof typeof stationData]?.system_name || ""
      );
    }
    if (stationData[language as keyof typeof stationData]?.description) {
      setValue(
        "description",
        stationData[language as keyof typeof stationData]?.description || ""
      );
    }
    if (
      stationData[language as keyof typeof stationData]?.latitude !== undefined
    ) {
      setValue(
        "latitude",
        stationData[language as keyof typeof stationData]?.latitude || 0
      );
    }
    if (
      stationData[language as keyof typeof stationData]?.longitude !== undefined
    ) {
      setValue(
        "longitude",
        stationData[language as keyof typeof stationData]?.longitude || 0
      );
    }
  }, [language, stationData, setValue]);

  useEffect(() => {
    setLanguageAction("ru");
  }, []);

  const [coordinates, setCoordinates] = useState("");

  const { id: stationId } = useParams<{ id: string }>();

  const latitudeContent = watch("latitude");
  const longitudeContent = watch("longitude");

  useEffect(() => {
    if (latitudeContent && longitudeContent) {
      setCoordinates(`${latitudeContent} ${longitudeContent}`);
    }
  }, [latitudeContent, longitudeContent]);

  const handleCoordinatesChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    setCoordinates(e.target.value);
    if (e.target.value) {
      const [lat, lon] = e.target.value
        .replace(/,/g, "") // Remove all commas from the string
        .split(" ")
        .map((s) => s.trim());
      setCoordinates(`${lat ?? 0} ${lon ?? 0}`);
      setValue("latitude", lat ?? 0);
      setValue("longitude", lon ?? 0);
    } else {
      setCoordinates("");
      setValue("latitude", "");
      setValue("longitude", "");
    }
  };

  const { autocompleteProps: cityAutocompleteProps } = useAutocomplete({
    resource: "city",
    onSearch: (value) => [
      {
        field: "name",
        operator: "contains",
        value,
      },
    ],

    meta: {
      headers: {
        "Accept-Language": "ru",
      },
    },
    queryOptions: {
      queryKey: ["city"],
    },
  });

  const cityId = watch("city_id");

  return (
    <Edit saveButtonProps={saveButtonProps}>
      <Box
        component="form"
        sx={{ display: "flex", flexDirection: "column" }}
        autoComplete="off"
      >
        <LanguageSwitch action={handleLanguageChange} />
        <TextField
          {...register("name", {
            required: "Это поле является обязательным",
          })}
          error={!!(errors as any)?.name}
          helperText={(errors as any)?.name?.message}
          margin="normal"
          fullWidth
          InputLabelProps={{ shrink: true }}
          type="text"
          label={"Название *"}
          name="name"
        />
        <TextField
          {...register("system_name", {
            required: "Это поле является обязательным",
          })}
          error={!!(errors as any)?.system_name}
          helperText={(errors as any)?.system_name?.message}
          margin="normal"
          fullWidth
          InputLabelProps={{ shrink: true }}
          type="text"
          label={"Системное название *"}
          name="system_name"
        />

        <input
          type="hidden"
          {...register("direction", { value: routeDirection })}
        />
        <Autocomplete
          options={directions}
          value={directions.find((el) => el.value == routeDirection)}
          onChange={(_, element) => {
            if (element) {
              setValue("direction", element.value);
              setRouteDirection(element.value);
            }
          }}
          renderInput={(params) => (
            <TextField
              {...params}
              label="Прямой/обратный маршрут"
              margin="normal"
              variant="outlined"
              error={!!errors.direction}
              helperText={(errors as any)?.direction?.message}
              required
            />
          )}
        />
        <TextField
          {...register("description", {
            // required: 'Это поле является обязательным',
          })}
          error={!!(errors as any)?.description}
          helperText={(errors as any)?.description?.message}
          margin="normal"
          fullWidth
          InputLabelProps={{ shrink: true }}
          type="text"
          label={"Описание"}
          name="description"
        />
        <TextField
          {...register("address", {
            // required: 'Это поле является обязательным',
          })}
          error={!!(errors as any)?.address}
          helperText={(errors as any)?.address?.message}
          margin="normal"
          fullWidth
          InputLabelProps={{ shrink: true }}
          type="text"
          label={"Адрес"}
          name="address"
        />

        <TextField
          value={coordinates}
          onChange={handleCoordinatesChange}
          error={!!(errors as any)?.latitude}
          helperText={(errors as any)?.latitude?.message}
          margin="normal"
          fullWidth
          InputLabelProps={{ shrink: true }}
          type="text"
          label={"Координаты *"}
        />
        <input
          type="hidden"
          {...register("latitude", {
            valueAsNumber: true,
            value: Number(coordinates.split(" ")[0]),
            setValueAs: (value) => {
              if (value === "") {
                return 0;
              }
              return Number(value);
            },
          })}
        />
        <input
          type="hidden"
          {...register("longitude", {
            valueAsNumber: true,
            value: Number(coordinates.split(" ")[1]),
            setValueAs: (value) => {
              if (value === "") {
                return 0;
              }
              return Number(value);
            },
          })}
        />

        <Controller
          control={control}
          name="city_id"
          rules={{ required: "Это поле является обязательным" }}
          defaultValue={null}
          render={({ field }) => (
            <Autocomplete
              {...cityAutocompleteProps}
              value={
                cityAutocompleteProps.options.find(
                  (option) => option.id === field.value
                ) || null
              }
              onChange={(_, value) => {
                field.onChange(value?.id || "");
              }}
              getOptionLabel={(item) => {
                return item ? item.name : "";
              }}
              isOptionEqualToValue={(option, value) => {
                return option.id === value?.id;
              }}
              filterOptions={(options, { inputValue }) => {
                return options.filter((option) =>
                  option.name.toLowerCase().includes(inputValue.toLowerCase())
                );
              }}
              renderInput={(params) => (
                <TextField
                  {...params}
                  label="Выберите город"
                  margin="normal"
                  variant="outlined"
                  error={!!errors.city_id}
                  helperText={(errors as any)?.city_id?.message}
                  required
                />
              )}
            />
          )}
        />
      </Box>

      {stationId && (
        <LinkedItems<SightItem>
          type="edit" // display and manage
          parentId={stationId}
          parentResource="station"
          childResource="sight"
          fields={sightFields}
          title="достопримечательности"
          dragAllowed={false}
          cityId={cityId}
        />
      )}
    </Edit>
  );
});
